import ReactDOM from 'react-dom';
import { jsx } from 'react/jsx-runtime';
import { createContext, useContext, useState, useRef, useEffect, isValidElement, useMemo } from 'react';
import { Designer } from 'sequential-workflow-designer';

var Presenter = /** @class */ (function () {
    function Presenter() {
    }
    Presenter.render = function (className, rootRef, element) {
        Presenter.tryDestroy(rootRef);
        var container = document.createElement('div');
        container.className = className;
        ReactDOM.render(element, container);
        return container;
    };
    Presenter.tryDestroy = function (rootRef) {
        if (rootRef.current) {
            var oldRoot = rootRef.current;
            rootRef.current = null;
            ReactDOM.unmountComponentAtNode(oldRoot);
        }
    };
    return Presenter;
}());

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

if (!window.sqdGlobalEditorContext) {
    window.sqdGlobalEditorContext = createContext(null);
}
var globalEditorContext = window.sqdGlobalEditorContext;
function useGlobalEditor() {
    var wrapper = useContext(globalEditorContext);
    if (!wrapper) {
        throw new Error('Cannot find global editor context');
    }
    return wrapper;
}
function GlobalEditorWrapperContext(props) {
    var _a = useState(function () { return createWrapper(); }), wrapper = _a[0], setWrapper = _a[1];
    function createWrapper() {
        return {
            properties: props.definition.properties,
            definition: props.definition,
            setProperty: setProperty
        };
    }
    function forward() {
        setWrapper(createWrapper());
    }
    function setProperty(name, value) {
        props.definition.properties[name] = value;
        props.context.notifyPropertiesChanged();
        forward();
    }
    return jsx(globalEditorContext.Provider, __assign({ value: wrapper }, { children: props.children }));
}

if (!window.sqdStepEditorContext) {
    window.sqdStepEditorContext = createContext(null);
}
var stepEditorContext = window.sqdStepEditorContext;
function useStepEditor() {
    var wrapper = useContext(stepEditorContext);
    if (!wrapper) {
        throw new Error('Cannot find step editor context');
    }
    return wrapper;
}
function StepEditorWrapperContext(props) {
    var _a = useState(function () { return createWrapper(); }), wrapper = _a[0], setWrapper = _a[1];
    function createWrapper() {
        return {
            id: props.step.id,
            type: props.step.type,
            componentType: props.step.componentType,
            name: props.step.name,
            properties: props.step.properties,
            step: props.step,
            definition: props.definition,
            setName: setName,
            setProperty: setProperty,
            notifyPropertiesChanged: notifyPropertiesChanged,
            notifyChildrenChanged: notifyChildrenChanged
        };
    }
    function forward() {
        setWrapper(createWrapper());
    }
    function setName(name) {
        props.step.name = name;
        notifyNameChanged();
    }
    function setProperty(name, value) {
        props.step.properties[name] = value;
        notifyPropertiesChanged();
    }
    function notifyNameChanged() {
        props.context.notifyNameChanged();
        forward();
    }
    function notifyPropertiesChanged() {
        props.context.notifyPropertiesChanged();
        forward();
    }
    function notifyChildrenChanged() {
        props.context.notifyChildrenChanged();
        forward();
    }
    return jsx(stepEditorContext.Provider, __assign({ value: wrapper }, { children: props.children }));
}

function wrapDefinition(value, isValid) {
    return {
        value: value,
        isValid: isValid
    };
}

var externalEditorClassName = 'sqd-editor-react';
function SequentialWorkflowDesigner(props) {
    var _a = useState(null), placeholder = _a[0], setPlaceholder = _a[1];
    var onDefinitionChangeRef = useRef(props.onDefinitionChange);
    var onSelectedStepIdChangedRef = useRef(props.onSelectedStepIdChanged);
    var onIsEditorCollapsedChangedRef = useRef(props.onIsEditorCollapsedChanged);
    var onIsToolboxCollapsedChangedRef = useRef(props.onIsToolboxCollapsedChanged);
    var globalEditorRef = useRef(props.globalEditor);
    var stepEditorRef = useRef(props.stepEditor);
    var controllerRef = useRef(props.controller);
    var customActionHandlerRef = useRef(props.customActionHandler);
    var designerRef = useRef(null);
    var editorRootRef = useRef(null);
    var definition = props.definition;
    var selectedStepId = props.selectedStepId;
    var isReadonly = props.isReadonly;
    var theme = props.theme;
    var undoStackSize = props.undoStackSize;
    var steps = props.stepsConfiguration;
    var validator = props.validatorConfiguration;
    var toolbox = props.toolboxConfiguration;
    var isEditorCollapsed = props.isEditorCollapsed;
    var isToolboxCollapsed = props.isToolboxCollapsed;
    var controlBar = props.controlBar;
    var contextMenu = props.contextMenu;
    var extensions = props.extensions;
    if (props.controlBar === undefined) {
        throw new Error('The "controlBar" property is not set');
    }
    function forwardDefinition() {
        if (designerRef.current) {
            var def = wrapDefinition(designerRef.current.getDefinition(), designerRef.current.isValid());
            onDefinitionChangeRef.current(def);
        }
    }
    function globalEditorProvider(def, context) {
        if (!globalEditorRef.current) {
            throw new Error('Global editor is not provided');
        }
        if (isValidElement(globalEditorRef.current)) {
            return Presenter.render(externalEditorClassName, editorRootRef, jsx(GlobalEditorWrapperContext, __assign({ definition: def, context: context }, { children: globalEditorRef.current })));
        }
        return globalEditorRef.current(def, context);
    }
    function stepEditorProvider(step, context, def) {
        if (!stepEditorRef.current) {
            throw new Error('Step editor is not provided');
        }
        if (isValidElement(stepEditorRef.current)) {
            return Presenter.render(externalEditorClassName, editorRootRef, jsx(StepEditorWrapperContext, __assign({ step: step, definition: def, context: context }, { children: stepEditorRef.current })));
        }
        return stepEditorRef.current(step, context, def);
    }
    function customActionHandler(action, step, sequence, context) {
        if (customActionHandlerRef.current) {
            customActionHandlerRef.current(action, step, sequence, context);
        }
    }
    function tryDestroy() {
        Presenter.tryDestroy(editorRootRef);
        if (controllerRef.current) {
            controllerRef.current.setDesigner(null);
        }
        if (designerRef.current) {
            designerRef.current.destroy();
            designerRef.current = null;
            // console.log('sqd: designer destroyed');
        }
    }
    useEffect(function () {
        onDefinitionChangeRef.current = props.onDefinitionChange;
    }, [props.onDefinitionChange]);
    useEffect(function () {
        onSelectedStepIdChangedRef.current = props.onSelectedStepIdChanged;
    }, [props.onSelectedStepIdChanged]);
    useEffect(function () {
        onIsEditorCollapsedChangedRef.current = props.onIsEditorCollapsedChanged;
    }, [props.onIsEditorCollapsedChanged]);
    useEffect(function () {
        onIsToolboxCollapsedChangedRef.current = props.onIsToolboxCollapsedChanged;
    }, [props.onIsToolboxCollapsedChanged]);
    useEffect(function () {
        globalEditorRef.current = props.globalEditor;
    }, [props.globalEditor]);
    useEffect(function () {
        stepEditorRef.current = props.stepEditor;
    }, [props.stepEditor]);
    useEffect(function () {
        customActionHandlerRef.current = props.customActionHandler;
    }, [props.customActionHandler]);
    useEffect(function () {
        if (!placeholder) {
            return;
        }
        if (designerRef.current) {
            var isNotChanged = definition.value === designerRef.current.getDefinition();
            if (isNotChanged) {
                if (selectedStepId !== undefined && selectedStepId !== designerRef.current.getSelectedStepId()) {
                    if (selectedStepId) {
                        designerRef.current.selectStepById(selectedStepId);
                    }
                    else {
                        designerRef.current.clearSelectedStep();
                    }
                    // console.log('sqd: selected step updated');
                }
                if (isReadonly !== undefined && isReadonly !== designerRef.current.isReadonly()) {
                    designerRef.current.setIsReadonly(isReadonly);
                    // console.log('sqd: isReadonly updated');
                }
                if (isToolboxCollapsed !== undefined && isToolboxCollapsed !== designerRef.current.isToolboxCollapsed()) {
                    designerRef.current.setIsToolboxCollapsed(isToolboxCollapsed);
                    // console.log('sqd: isToolboxCollapsed updated');
                }
                if (isEditorCollapsed !== undefined && isEditorCollapsed !== designerRef.current.isEditorCollapsed()) {
                    designerRef.current.setIsEditorCollapsed(isEditorCollapsed);
                    // console.log('sqd: isEditorCollapsed updated');
                }
                return;
            }
            tryDestroy();
        }
        var designer = Designer.create(placeholder, definition.value, {
            theme: theme,
            undoStackSize: undoStackSize,
            toolbox: toolbox
                ? __assign(__assign({}, toolbox), { isCollapsed: isToolboxCollapsed }) : false,
            steps: steps,
            validator: validator,
            controlBar: controlBar,
            contextMenu: contextMenu,
            editors: globalEditorRef.current && stepEditorRef.current
                ? {
                    isCollapsed: isEditorCollapsed,
                    globalEditorProvider: globalEditorProvider,
                    stepEditorProvider: stepEditorProvider
                }
                : false,
            customActionHandler: customActionHandlerRef.current && customActionHandler,
            extensions: extensions
        });
        if (controllerRef.current) {
            controllerRef.current.setDesigner(designer);
        }
        if (selectedStepId) {
            designer.selectStepById(selectedStepId);
        }
        if (isReadonly) {
            designer.setIsReadonly(isReadonly);
        }
        // console.log('sqd: designer rendered');
        designer.onReady.subscribe(forwardDefinition);
        designer.onDefinitionChanged.subscribe(forwardDefinition);
        designer.onSelectedStepIdChanged.subscribe(function (stepId) {
            if (onSelectedStepIdChangedRef.current) {
                onSelectedStepIdChangedRef.current(stepId);
            }
        });
        designer.onIsToolboxCollapsedChanged.subscribe(function (isCollapsed) {
            if (onIsToolboxCollapsedChangedRef.current) {
                onIsToolboxCollapsedChangedRef.current(isCollapsed);
            }
        });
        designer.onIsEditorCollapsedChanged.subscribe(function (isCollapsed) {
            if (onIsEditorCollapsedChangedRef.current) {
                onIsEditorCollapsedChangedRef.current(isCollapsed);
            }
        });
        designerRef.current = designer;
    }, [
        placeholder,
        definition,
        selectedStepId,
        isReadonly,
        theme,
        undoStackSize,
        toolbox,
        isToolboxCollapsed,
        isEditorCollapsed,
        contextMenu,
        controlBar,
        steps,
        validator,
        extensions
    ]);
    useEffect(function () {
        return tryDestroy;
    }, []);
    return jsx("div", { ref: setPlaceholder, "data-testid": "designer", className: "sqd-designer-react" });
}

var SequentialWorkflowDesignerController = /** @class */ (function () {
    function SequentialWorkflowDesignerController() {
        var _this = this;
        this.designer = null;
        /**
         * @description Moves the viewport to the step with the animation.
         */
        this.moveViewportToStep = function (stepId) {
            _this.getDesigner().moveViewportToStep(stepId);
        };
        /**
         * @description Updates all badges.
         */
        this.updateBadges = function () {
            _this.getDesigner().updateBadges();
        };
        /**
         * @description Rerender the root component and all its children.
         */
        this.updateRootComponent = function () {
            _this.getDesigner().updateRootComponent();
        };
        // Nothing...
    }
    SequentialWorkflowDesignerController.create = function () {
        return new SequentialWorkflowDesignerController();
    };
    /**
     * @returns `true` if the controller is ready to be used, `false` otherwise.
     */
    SequentialWorkflowDesignerController.prototype.isReady = function () {
        return !!this.designer;
    };
    SequentialWorkflowDesignerController.prototype.setDesigner = function (designer) {
        if (designer && this.designer) {
            throw new Error('Designer is already set');
        }
        this.designer = designer;
    };
    SequentialWorkflowDesignerController.prototype.getDesigner = function () {
        if (!this.designer) {
            throw new Error('Designer is not ready yet');
        }
        return this.designer;
    };
    return SequentialWorkflowDesignerController;
}());
function useSequentialWorkflowDesignerController(deps) {
    return useMemo(function () { return SequentialWorkflowDesignerController.create(); }, deps || []);
}

export { GlobalEditorWrapperContext, Presenter, SequentialWorkflowDesigner, SequentialWorkflowDesignerController, StepEditorWrapperContext, useGlobalEditor, useSequentialWorkflowDesignerController, useStepEditor, wrapDefinition };
